"""
Views for the OPAL referral portal plugin
"""
from django.views.generic import TemplateView

from opal.core.views import LoginRequiredMixin


class ReferralIndexView(LoginRequiredMixin, TemplateView):
    """
    Main entrypoint into the referral portal service.

    Lists our referral routes.
    """
    template_name = 'referral/index.html'


class ReferralTemplateView(TemplateView):
    def dispatch(self, *args, **kwargs):
        self.name = kwargs['name']
        return super(ReferralTemplateView, self).dispatch(*args, **kwargs)

    def get_template_names(self, *args, **kwargs):
        return ['referral/'+self.name, 'referral/referral.html']

    def get_context_data(self, *args, **kwargs):
        context = super(ReferralTemplateView, self).get_context_data(**kwargs)
        from referral import ReferralRoute
        context['referral_routes'] = ReferralRoute.list()
        return context



"""
Establishing referral routes
"""
from django.conf import settings

from opal.utils import stringport, camelcase_to_underscore, _itersubclasses
from opal.core.schemas import serialize_model

# So we only do it once
IMPORTED_FROM_APPS = False


def import_from_apps():
    """
    Iterate through installed apps attempting to import app.wardrounds
    This way we allow our implementation, or plugins, to define their
    own ward rounds.
    """
    for app in settings.INSTALLED_APPS:
        try:
            stringport(app + '.referrals')
        except ImportError:
            pass # not a problem
    global IMPORTED_FROM_APPS
    IMPORTED_FROM_APPS = True
    return


class ReferralRoute(object):
    """
    Base Referral Route class - individal referral routes should override this.
    """
    name         = 'Please name me Larry!'
    description  = 'Please describe me Larry!'
    target_teams = []
    target_category = None
    success_link = None
    page_title = None
    verb = 'Refer'
    progressive_verb = 'Referring'
    past_verb = 'Referred'
    create_new_episode = True
    additional_models = []

    @classmethod
    def get(klass, name):
        """
        Return a specific referral route by slug
        """
        for sub in klass.list():
            if sub.slug() == name:
                return sub

    @classmethod
    def list(klass):
        """
        Return a list of all ward rounds
        """

        if not IMPORTED_FROM_APPS:
            import_from_apps()
        return _itersubclasses(klass)

    @classmethod
    def slug(klass):
        return camelcase_to_underscore(klass.name).replace(' ', '')

    @classmethod
    def to_dict(klass):
        """
        Serialise referral routes for the client
        """
        return dict(
            name=klass.name,
            description=klass.description,
            slug=klass.slug(),
            verb=klass.verb,
            progressive_verb=klass.progressive_verb,
            past_verb=klass.past_verb,
            page_title=klass.page_title,
            additional_models=[
                serialize_model(m) for m in klass.additional_models
            ]
        )

    def post_create(self, episode, user):
        """
        Subclasses should override this method to manipulate an episode
        immediately after it has been created.
        """
        return

    def get_success_link(self, episode):
        return self.success_link



"""
API for the OPAL referral portal.
"""
from rest_framework import status
from rest_framework.response import Response
from rest_framework.viewsets import ViewSet

from opal.models import Patient

class ReferralViewSet(ViewSet):

    def list(self, request):
        return Response(self.referral.to_dict())

    def create(self, request):
        hospital_number = request.data['hospital_number']
        patient, created = Patient.objects.get_or_create(
            demographics__hospital_number=hospital_number
        )
        demographics = patient.demographics_set.get()
        target_teams = set(self.referral.target_teams)
        if created:
            demographics.hospital_number = hospital_number
            demographics.save()

        if 'demographics' in request.data:
            demographics.update_from_dict(
                request.data['demographics'], request.user
            )

        # we should never, even accidentally have a patient without an episode
        if self.referral.create_new_episode or not patient.episode_set.count():
            episode = patient.create_episode()
        else:
            episode = patient.episode_set.order_by("-created").first()
            target_teams = target_teams.union(episode.get_tag_names(request.user))

        for additional_model in self.referral.additional_models:
            model_name = additional_model.__name__.lower()

            if model_name in request.data:
                new_model = additional_model()
                data_dict = request.data[model_name]
                data_dict["episode_id"] = episode.id
                new_model.update_from_dict(data_dict, request.user)

        if self.referral.target_category:
            episode.category_name = self.referral.target_category
            episode.save()
        episode.set_tag_names(list(target_teams), request.user)
        referral = self.referral()
        referral.post_create(episode, request.user)
        success_link = referral.get_success_link(episode)
        self.referral().post_create(episode, request.user)
        return Response({'success_link': success_link}, status.HTTP_201_CREATED)


def viewsets():
    """
    Return our api viewsets
    """
    from referral import ReferralRoute

    apis = []
    for route in ReferralRoute.list():
        class RouteAPI(ReferralViewSet):
            referral = route
            base_name = 'referral/'+route.slug()

        apis.append(('referral/'+route.slug(), RouteAPI))
    return apis



"""
Plugin definition
"""
from django.conf import settings
from opal.core import plugins

from referral import api
from referral.urls import urlpatterns
from referral.routes import ReferralRoute

DISPLAY_MENU = getattr(settings, 'REFERRAL_MENU_ITEM', True)
if DISPLAY_MENU:
    menuitems = [
        dict(
            href='/referrals/#/', display='Referrals', icon='fa fa-mail-forward',
            activepattern='/referrals', index=2)
    ]
else:
    menuitems = []

class ReferralPortalPlugin(plugins.OpalPlugin):
    """
    Main entrypoint to expose this plugin to the host
    OPAL instance.
    """
    urls = urlpatterns
    javascripts = {
        'opal.referral': [
            'js/referral/app.js',
            'js/referral/services/referral_route_loader.js',
            'js/referral/controllers/list.js',
            'js/referral/controllers/referral.js',
            # 'js/referral/controllers/episode_detail.js',
        ]
    }
    menuitems = menuitems
    apis = api.viewsets()

plugins.register(ReferralPortalPlugin)



import os
from setuptools import setup

long_desc = """
OPAL Referral is a plugin for the OPAL web framework that provides functionality for making
referrals between hospital teams or to hospital departments/clinics.

Source code and documentation available at https://github.com/openhealthcare/opal-referral/
"""

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='opal-referral',
    version='0.2.0',
    packages=['referral'],
    include_package_data=True,
    license='GPL3',
    description='OPAL Referral Portal plugin ',
    long_description=long_desc,
    url='http://opal.openhealthcare.org.uk/',
    author='Open Health Care UK',
    author_email='hello@openhealthcare.org.uk',
)



"""
Urls for the OPAL referrals plugin
"""
from django.conf.urls import patterns, url

from referral import views

urlpatterns = patterns(
    '',
    url('^referrals/$', views.ReferralIndexView.as_view()),
    url(r'^referral/templates/(?P<name>[a-z_]+.html)$',
        views.ReferralTemplateView.as_view()),
)
